/****************************************************************************
** Meta object code from reading C++ file 'qconnection_local_backend_p.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.2)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../qtremoteobjects/src/remoteobjects/qconnection_local_backend_p.h"
#include <QtRemoteObjects/qremoteobjectnode.h>
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'qconnection_local_backend_p.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.2. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN13LocalClientIoE_t {};
} // unnamed namespace

template <> constexpr inline auto LocalClientIo::qt_create_metaobjectdata<qt_meta_tag_ZN13LocalClientIoE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "LocalClientIo",
        "onError",
        "",
        "QLocalSocket::LocalSocketError",
        "error",
        "onStateChanged",
        "QLocalSocket::LocalSocketState",
        "state"
    };

    QtMocHelpers::UintData qt_methods {
        // Slot 'onError'
        QtMocHelpers::SlotData<void(QLocalSocket::LocalSocketError)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
        // Slot 'onStateChanged'
        QtMocHelpers::SlotData<void(QLocalSocket::LocalSocketState)>(5, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 6, 7 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<LocalClientIo, qt_meta_tag_ZN13LocalClientIoE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject LocalClientIo::staticMetaObject = { {
    QMetaObject::SuperData::link<QtROClientIoDevice::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN13LocalClientIoE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN13LocalClientIoE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN13LocalClientIoE_t>.metaTypes,
    nullptr
} };

void LocalClientIo::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<LocalClientIo *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->onError((*reinterpret_cast<std::add_pointer_t<QLocalSocket::LocalSocketError>>(_a[1]))); break;
        case 1: _t->onStateChanged((*reinterpret_cast<std::add_pointer_t<QLocalSocket::LocalSocketState>>(_a[1]))); break;
        default: ;
        }
    }
}

const QMetaObject *LocalClientIo::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *LocalClientIo::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN13LocalClientIoE_t>.strings))
        return static_cast<void*>(this);
    return QtROClientIoDevice::qt_metacast(_clname);
}

int LocalClientIo::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QtROClientIoDevice::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 2)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 2;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 2)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 2;
    }
    return _id;
}
namespace {
struct qt_meta_tag_ZN21AbstractLocalClientIoE_t {};
} // unnamed namespace

template <> constexpr inline auto AbstractLocalClientIo::qt_create_metaobjectdata<qt_meta_tag_ZN21AbstractLocalClientIoE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "AbstractLocalClientIo"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<AbstractLocalClientIo, qt_meta_tag_ZN21AbstractLocalClientIoE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject AbstractLocalClientIo::staticMetaObject = { {
    QMetaObject::SuperData::link<LocalClientIo::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN21AbstractLocalClientIoE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN21AbstractLocalClientIoE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN21AbstractLocalClientIoE_t>.metaTypes,
    nullptr
} };

void AbstractLocalClientIo::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<AbstractLocalClientIo *>(_o);
    (void)_t;
    (void)_c;
    (void)_id;
    (void)_a;
}

const QMetaObject *AbstractLocalClientIo::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *AbstractLocalClientIo::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN21AbstractLocalClientIoE_t>.strings))
        return static_cast<void*>(this);
    return LocalClientIo::qt_metacast(_clname);
}

int AbstractLocalClientIo::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = LocalClientIo::qt_metacall(_c, _id, _a);
    return _id;
}
namespace {
struct qt_meta_tag_ZN13LocalServerIoE_t {};
} // unnamed namespace

template <> constexpr inline auto LocalServerIo::qt_create_metaobjectdata<qt_meta_tag_ZN13LocalServerIoE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "LocalServerIo"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<LocalServerIo, qt_meta_tag_ZN13LocalServerIoE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject LocalServerIo::staticMetaObject = { {
    QMetaObject::SuperData::link<QtROServerIoDevice::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN13LocalServerIoE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN13LocalServerIoE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN13LocalServerIoE_t>.metaTypes,
    nullptr
} };

void LocalServerIo::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<LocalServerIo *>(_o);
    (void)_t;
    (void)_c;
    (void)_id;
    (void)_a;
}

const QMetaObject *LocalServerIo::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *LocalServerIo::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN13LocalServerIoE_t>.strings))
        return static_cast<void*>(this);
    return QtROServerIoDevice::qt_metacast(_clname);
}

int LocalServerIo::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QtROServerIoDevice::qt_metacall(_c, _id, _a);
    return _id;
}
namespace {
struct qt_meta_tag_ZN15LocalServerImplE_t {};
} // unnamed namespace

template <> constexpr inline auto LocalServerImpl::qt_create_metaobjectdata<qt_meta_tag_ZN15LocalServerImplE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "LocalServerImpl"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<LocalServerImpl, qt_meta_tag_ZN15LocalServerImplE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject LocalServerImpl::staticMetaObject = { {
    QMetaObject::SuperData::link<QConnectionAbstractServer::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN15LocalServerImplE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN15LocalServerImplE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN15LocalServerImplE_t>.metaTypes,
    nullptr
} };

void LocalServerImpl::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<LocalServerImpl *>(_o);
    (void)_t;
    (void)_c;
    (void)_id;
    (void)_a;
}

const QMetaObject *LocalServerImpl::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *LocalServerImpl::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN15LocalServerImplE_t>.strings))
        return static_cast<void*>(this);
    return QConnectionAbstractServer::qt_metacast(_clname);
}

int LocalServerImpl::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QConnectionAbstractServer::qt_metacall(_c, _id, _a);
    return _id;
}
namespace {
struct qt_meta_tag_ZN23AbstractLocalServerImplE_t {};
} // unnamed namespace

template <> constexpr inline auto AbstractLocalServerImpl::qt_create_metaobjectdata<qt_meta_tag_ZN23AbstractLocalServerImplE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "AbstractLocalServerImpl"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<AbstractLocalServerImpl, qt_meta_tag_ZN23AbstractLocalServerImplE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject AbstractLocalServerImpl::staticMetaObject = { {
    QMetaObject::SuperData::link<LocalServerImpl::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN23AbstractLocalServerImplE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN23AbstractLocalServerImplE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN23AbstractLocalServerImplE_t>.metaTypes,
    nullptr
} };

void AbstractLocalServerImpl::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<AbstractLocalServerImpl *>(_o);
    (void)_t;
    (void)_c;
    (void)_id;
    (void)_a;
}

const QMetaObject *AbstractLocalServerImpl::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *AbstractLocalServerImpl::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN23AbstractLocalServerImplE_t>.strings))
        return static_cast<void*>(this);
    return LocalServerImpl::qt_metacast(_clname);
}

int AbstractLocalServerImpl::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = LocalServerImpl::qt_metacall(_c, _id, _a);
    return _id;
}
QT_WARNING_POP
