/****************************************************************************
** Meta object code from reading C++ file 'qremoteobjectnode.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.2)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../qtremoteobjects/src/remoteobjects/qremoteobjectnode.h"
#include <QtRemoteObjects/qremoteobjectnode.h>
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'qremoteobjectnode.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.2. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN35QRemoteObjectAbstractPersistedStoreE_t {};
} // unnamed namespace

template <> constexpr inline auto QRemoteObjectAbstractPersistedStore::qt_create_metaobjectdata<qt_meta_tag_ZN35QRemoteObjectAbstractPersistedStoreE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "QRemoteObjectAbstractPersistedStore"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<QRemoteObjectAbstractPersistedStore, qt_meta_tag_ZN35QRemoteObjectAbstractPersistedStoreE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject QRemoteObjectAbstractPersistedStore::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN35QRemoteObjectAbstractPersistedStoreE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN35QRemoteObjectAbstractPersistedStoreE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN35QRemoteObjectAbstractPersistedStoreE_t>.metaTypes,
    nullptr
} };

void QRemoteObjectAbstractPersistedStore::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<QRemoteObjectAbstractPersistedStore *>(_o);
    (void)_t;
    (void)_c;
    (void)_id;
    (void)_a;
}

const QMetaObject *QRemoteObjectAbstractPersistedStore::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QRemoteObjectAbstractPersistedStore::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN35QRemoteObjectAbstractPersistedStoreE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int QRemoteObjectAbstractPersistedStore::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    return _id;
}
namespace {
struct qt_meta_tag_ZN17QRemoteObjectNodeE_t {};
} // unnamed namespace

template <> constexpr inline auto QRemoteObjectNode::qt_create_metaobjectdata<qt_meta_tag_ZN17QRemoteObjectNodeE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "QRemoteObjectNode",
        "remoteObjectAdded",
        "",
        "QRemoteObjectSourceLocation",
        "remoteObjectRemoved",
        "error",
        "QRemoteObjectNode::ErrorCode",
        "errorCode",
        "heartbeatIntervalChanged",
        "heartbeatInterval",
        "connectToNode",
        "QUrl",
        "address",
        "registryUrl",
        "persistedStore",
        "QRemoteObjectAbstractPersistedStore*",
        "ErrorCode",
        "NoError",
        "RegistryNotAcquired",
        "RegistryAlreadyHosted",
        "NodeIsNoServer",
        "ServerAlreadyCreated",
        "UnintendedRegistryHosting",
        "OperationNotValidOnClientNode",
        "SourceNotRegistered",
        "MissingObjectName",
        "HostUrlInvalid",
        "ProtocolMismatch",
        "ListenFailed",
        "SocketAccessError"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'remoteObjectAdded'
        QtMocHelpers::SignalData<void(const QRemoteObjectSourceLocation &)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 2 },
        }}),
        // Signal 'remoteObjectRemoved'
        QtMocHelpers::SignalData<void(const QRemoteObjectSourceLocation &)>(4, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 2 },
        }}),
        // Signal 'error'
        QtMocHelpers::SignalData<void(QRemoteObjectNode::ErrorCode)>(5, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 6, 7 },
        }}),
        // Signal 'heartbeatIntervalChanged'
        QtMocHelpers::SignalData<void(int)>(8, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 9 },
        }}),
        // Method 'connectToNode'
        QtMocHelpers::MethodData<bool(const QUrl &)>(10, 2, QMC::AccessPublic, QMetaType::Bool, {{
            { 0x80000000 | 11, 12 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'registryUrl'
        QtMocHelpers::PropertyData<QUrl>(13, 0x80000000 | 11, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet),
        // property 'persistedStore'
        QtMocHelpers::PropertyData<QRemoteObjectAbstractPersistedStore*>(14, 0x80000000 | 15, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet),
        // property 'heartbeatInterval'
        QtMocHelpers::PropertyData<int>(9, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 3),
    };
    QtMocHelpers::UintData qt_enums {
        // enum 'ErrorCode'
        QtMocHelpers::EnumData<enum ErrorCode>(16, 16, QMC::EnumFlags{}).add({
            {   17, ErrorCode::NoError },
            {   18, ErrorCode::RegistryNotAcquired },
            {   19, ErrorCode::RegistryAlreadyHosted },
            {   20, ErrorCode::NodeIsNoServer },
            {   21, ErrorCode::ServerAlreadyCreated },
            {   22, ErrorCode::UnintendedRegistryHosting },
            {   23, ErrorCode::OperationNotValidOnClientNode },
            {   24, ErrorCode::SourceNotRegistered },
            {   25, ErrorCode::MissingObjectName },
            {   26, ErrorCode::HostUrlInvalid },
            {   27, ErrorCode::ProtocolMismatch },
            {   28, ErrorCode::ListenFailed },
            {   29, ErrorCode::SocketAccessError },
        }),
    };
    return QtMocHelpers::metaObjectData<QRemoteObjectNode, qt_meta_tag_ZN17QRemoteObjectNodeE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject QRemoteObjectNode::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN17QRemoteObjectNodeE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN17QRemoteObjectNodeE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN17QRemoteObjectNodeE_t>.metaTypes,
    nullptr
} };

void QRemoteObjectNode::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<QRemoteObjectNode *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->remoteObjectAdded((*reinterpret_cast<std::add_pointer_t<QRemoteObjectSourceLocation>>(_a[1]))); break;
        case 1: _t->remoteObjectRemoved((*reinterpret_cast<std::add_pointer_t<QRemoteObjectSourceLocation>>(_a[1]))); break;
        case 2: _t->error((*reinterpret_cast<std::add_pointer_t<QRemoteObjectNode::ErrorCode>>(_a[1]))); break;
        case 3: _t->heartbeatIntervalChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 4: { bool _r = _t->connectToNode((*reinterpret_cast<std::add_pointer_t<QUrl>>(_a[1])));
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 0:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QRemoteObjectSourceLocation >(); break;
            }
            break;
        case 1:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QRemoteObjectSourceLocation >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (QRemoteObjectNode::*)(const QRemoteObjectSourceLocation & )>(_a, &QRemoteObjectNode::remoteObjectAdded, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (QRemoteObjectNode::*)(const QRemoteObjectSourceLocation & )>(_a, &QRemoteObjectNode::remoteObjectRemoved, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (QRemoteObjectNode::*)(QRemoteObjectNode::ErrorCode )>(_a, &QRemoteObjectNode::error, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (QRemoteObjectNode::*)(int )>(_a, &QRemoteObjectNode::heartbeatIntervalChanged, 3))
            return;
    }
    if (_c == QMetaObject::RegisterPropertyMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 1:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QRemoteObjectAbstractPersistedStore* >(); break;
        }
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<QUrl*>(_v) = _t->registryUrl(); break;
        case 1: *reinterpret_cast<QRemoteObjectAbstractPersistedStore**>(_v) = _t->persistedStore(); break;
        case 2: *reinterpret_cast<int*>(_v) = _t->heartbeatInterval(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setRegistryUrl(*reinterpret_cast<QUrl*>(_v)); break;
        case 1: _t->setPersistedStore(*reinterpret_cast<QRemoteObjectAbstractPersistedStore**>(_v)); break;
        case 2: _t->setHeartbeatInterval(*reinterpret_cast<int*>(_v)); break;
        default: break;
        }
    }
}

const QMetaObject *QRemoteObjectNode::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QRemoteObjectNode::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN17QRemoteObjectNodeE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int QRemoteObjectNode::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 5)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 5;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 5)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 5;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 3;
    }
    return _id;
}

// SIGNAL 0
void QRemoteObjectNode::remoteObjectAdded(const QRemoteObjectSourceLocation & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void QRemoteObjectNode::remoteObjectRemoved(const QRemoteObjectSourceLocation & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}

// SIGNAL 2
void QRemoteObjectNode::error(QRemoteObjectNode::ErrorCode _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1);
}

// SIGNAL 3
void QRemoteObjectNode::heartbeatIntervalChanged(int _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1);
}
namespace {
struct qt_meta_tag_ZN21QRemoteObjectHostBaseE_t {};
} // unnamed namespace

template <> constexpr inline auto QRemoteObjectHostBase::qt_create_metaobjectdata<qt_meta_tag_ZN21QRemoteObjectHostBaseE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "QRemoteObjectHostBase",
        "enableRemoting",
        "",
        "object",
        "name",
        "disableRemoting",
        "remoteObject",
        "AllowedSchemas",
        "BuiltInSchemasOnly",
        "AllowExternalRegistration"
    };

    QtMocHelpers::UintData qt_methods {
        // Method 'enableRemoting'
        QtMocHelpers::MethodData<bool(QObject *, const QString &)>(1, 2, QMC::AccessPublic, QMetaType::Bool, {{
            { QMetaType::QObjectStar, 3 }, { QMetaType::QString, 4 },
        }}),
        // Method 'enableRemoting'
        QtMocHelpers::MethodData<bool(QObject *)>(1, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Bool, {{
            { QMetaType::QObjectStar, 3 },
        }}),
        // Method 'disableRemoting'
        QtMocHelpers::MethodData<bool(QObject *)>(5, 2, QMC::AccessPublic, QMetaType::Bool, {{
            { QMetaType::QObjectStar, 6 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
        // enum 'AllowedSchemas'
        QtMocHelpers::EnumData<enum AllowedSchemas>(7, 7, QMC::EnumFlags{}).add({
            {    8, AllowedSchemas::BuiltInSchemasOnly },
            {    9, AllowedSchemas::AllowExternalRegistration },
        }),
    };
    return QtMocHelpers::metaObjectData<QRemoteObjectHostBase, qt_meta_tag_ZN21QRemoteObjectHostBaseE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject QRemoteObjectHostBase::staticMetaObject = { {
    QMetaObject::SuperData::link<QRemoteObjectNode::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN21QRemoteObjectHostBaseE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN21QRemoteObjectHostBaseE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN21QRemoteObjectHostBaseE_t>.metaTypes,
    nullptr
} };

void QRemoteObjectHostBase::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<QRemoteObjectHostBase *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: { bool _r = _t->enableRemoting((*reinterpret_cast<std::add_pointer_t<QObject*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])));
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 1: { bool _r = _t->enableRemoting((*reinterpret_cast<std::add_pointer_t<QObject*>>(_a[1])));
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 2: { bool _r = _t->disableRemoting((*reinterpret_cast<std::add_pointer_t<QObject*>>(_a[1])));
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        default: ;
        }
    }
}

const QMetaObject *QRemoteObjectHostBase::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QRemoteObjectHostBase::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN21QRemoteObjectHostBaseE_t>.strings))
        return static_cast<void*>(this);
    return QRemoteObjectNode::qt_metacast(_clname);
}

int QRemoteObjectHostBase::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QRemoteObjectNode::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 3)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 3;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 3)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 3;
    }
    return _id;
}
namespace {
struct qt_meta_tag_ZN17QRemoteObjectHostE_t {};
} // unnamed namespace

template <> constexpr inline auto QRemoteObjectHost::qt_create_metaobjectdata<qt_meta_tag_ZN17QRemoteObjectHostE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "QRemoteObjectHost",
        "hostUrlChanged",
        "",
        "hostUrl",
        "QUrl"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'hostUrlChanged'
        QtMocHelpers::SignalData<void()>(1, 2, QMC::AccessPublic, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'hostUrl'
        QtMocHelpers::PropertyData<QUrl>(3, 0x80000000 | 4, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 0),
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<QRemoteObjectHost, qt_meta_tag_ZN17QRemoteObjectHostE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject QRemoteObjectHost::staticMetaObject = { {
    QMetaObject::SuperData::link<QRemoteObjectHostBase::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN17QRemoteObjectHostE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN17QRemoteObjectHostE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN17QRemoteObjectHostE_t>.metaTypes,
    nullptr
} };

void QRemoteObjectHost::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<QRemoteObjectHost *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->hostUrlChanged(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (QRemoteObjectHost::*)()>(_a, &QRemoteObjectHost::hostUrlChanged, 0))
            return;
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<QUrl*>(_v) = _t->hostUrl(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setHostUrl(*reinterpret_cast<QUrl*>(_v)); break;
        default: break;
        }
    }
}

const QMetaObject *QRemoteObjectHost::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QRemoteObjectHost::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN17QRemoteObjectHostE_t>.strings))
        return static_cast<void*>(this);
    return QRemoteObjectHostBase::qt_metacast(_clname);
}

int QRemoteObjectHost::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QRemoteObjectHostBase::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 1)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 1;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 1)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 1;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 1;
    }
    return _id;
}

// SIGNAL 0
void QRemoteObjectHost::hostUrlChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}
namespace {
struct qt_meta_tag_ZN25QRemoteObjectRegistryHostE_t {};
} // unnamed namespace

template <> constexpr inline auto QRemoteObjectRegistryHost::qt_create_metaobjectdata<qt_meta_tag_ZN25QRemoteObjectRegistryHostE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "QRemoteObjectRegistryHost"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<QRemoteObjectRegistryHost, qt_meta_tag_ZN25QRemoteObjectRegistryHostE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject QRemoteObjectRegistryHost::staticMetaObject = { {
    QMetaObject::SuperData::link<QRemoteObjectHostBase::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN25QRemoteObjectRegistryHostE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN25QRemoteObjectRegistryHostE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN25QRemoteObjectRegistryHostE_t>.metaTypes,
    nullptr
} };

void QRemoteObjectRegistryHost::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<QRemoteObjectRegistryHost *>(_o);
    (void)_t;
    (void)_c;
    (void)_id;
    (void)_a;
}

const QMetaObject *QRemoteObjectRegistryHost::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QRemoteObjectRegistryHost::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN25QRemoteObjectRegistryHostE_t>.strings))
        return static_cast<void*>(this);
    return QRemoteObjectHostBase::qt_metacast(_clname);
}

int QRemoteObjectRegistryHost::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QRemoteObjectHostBase::qt_metacall(_c, _id, _a);
    return _id;
}
QT_WARNING_POP
