/****************************************************************************
** Meta object code from reading C++ file 'qtremoteobjectglobal.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.2)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../qtremoteobjects/src/remoteobjects/qtremoteobjectglobal.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'qtremoteobjectglobal.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.2. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN15QtRemoteObjectsE_t {};
} // unnamed namespace

template <> constexpr inline auto QtRemoteObjects::qt_create_metaobjectdata<qt_meta_tag_ZN15QtRemoteObjectsE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "QtRemoteObjects",
        "QRemoteObjectPacketTypeEnum",
        "Invalid",
        "Handshake",
        "InitPacket",
        "InitDynamicPacket",
        "AddObject",
        "RemoveObject",
        "InvokePacket",
        "InvokeReplyPacket",
        "PropertyChangePacket",
        "ObjectList",
        "Ping",
        "Pong",
        "InitialAction",
        "FetchRootSize",
        "PrefetchData"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
        // enum 'QRemoteObjectPacketTypeEnum'
        QtMocHelpers::EnumData<QRemoteObjectPacketTypeEnum>(1, 1, QMC::EnumFlags{}).add({
            {    2, QRemoteObjectPacketTypeEnum::Invalid },
            {    3, QRemoteObjectPacketTypeEnum::Handshake },
            {    4, QRemoteObjectPacketTypeEnum::InitPacket },
            {    5, QRemoteObjectPacketTypeEnum::InitDynamicPacket },
            {    6, QRemoteObjectPacketTypeEnum::AddObject },
            {    7, QRemoteObjectPacketTypeEnum::RemoveObject },
            {    8, QRemoteObjectPacketTypeEnum::InvokePacket },
            {    9, QRemoteObjectPacketTypeEnum::InvokeReplyPacket },
            {   10, QRemoteObjectPacketTypeEnum::PropertyChangePacket },
            {   11, QRemoteObjectPacketTypeEnum::ObjectList },
            {   12, QRemoteObjectPacketTypeEnum::Ping },
            {   13, QRemoteObjectPacketTypeEnum::Pong },
        }),
        // enum 'InitialAction'
        QtMocHelpers::EnumData<InitialAction>(14, 14, QMC::EnumFlags{}).add({
            {   15, InitialAction::FetchRootSize },
            {   16, InitialAction::PrefetchData },
        }),
    };
    return QtMocHelpers::metaObjectData<void, qt_meta_tag_ZN15QtRemoteObjectsE_t>(QMC::PropertyAccessInStaticMetaCall, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}

static constexpr auto qt_staticMetaObjectContent_ZN15QtRemoteObjectsE =
    QtRemoteObjects::qt_create_metaobjectdata<qt_meta_tag_ZN15QtRemoteObjectsE_t>();
static constexpr auto qt_staticMetaObjectStaticContent_ZN15QtRemoteObjectsE =
    qt_staticMetaObjectContent_ZN15QtRemoteObjectsE.staticData;
static constexpr auto qt_staticMetaObjectRelocatingContent_ZN15QtRemoteObjectsE =
    qt_staticMetaObjectContent_ZN15QtRemoteObjectsE.relocatingData;

Q_CONSTINIT const QMetaObject QtRemoteObjects::staticMetaObject = { {
    nullptr,
    qt_staticMetaObjectStaticContent_ZN15QtRemoteObjectsE.stringdata,
    qt_staticMetaObjectStaticContent_ZN15QtRemoteObjectsE.data,
    nullptr,
    nullptr,
    qt_staticMetaObjectRelocatingContent_ZN15QtRemoteObjectsE.metaTypes,
    nullptr
} };

QT_WARNING_POP
