/****************************************************************************
** Meta object code from reading C++ file 'qremoteobjectsqml_p.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.2)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../qtremoteobjects/src/remoteobjectsqml/qremoteobjectsqml_p.h"
#include <QtRemoteObjects/qremoteobjectnode.h>
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'qremoteobjectsqml_p.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.2. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN18QtQmlRemoteObjectsE_t {};
} // unnamed namespace

template <> constexpr inline auto QtQmlRemoteObjects::qt_create_metaobjectdata<qt_meta_tag_ZN18QtQmlRemoteObjectsE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "QtQmlRemoteObjects",
        "QML.Element",
        "QtRemoteObjects",
        "QML.Singleton",
        "true",
        "QML.AddedInVersion",
        "1294",
        "watch",
        "QJSValue",
        "",
        "QRemoteObjectPendingCall",
        "reply",
        "timeout"
    };

    QtMocHelpers::UintData qt_methods {
        // Method 'watch'
        QtMocHelpers::MethodData<QJSValue(const QRemoteObjectPendingCall &, int)>(7, 9, QMC::AccessPublic, 0x80000000 | 8, {{
            { 0x80000000 | 10, 11 }, { QMetaType::Int, 12 },
        }}),
        // Method 'watch'
        QtMocHelpers::MethodData<QJSValue(const QRemoteObjectPendingCall &)>(7, 9, QMC::AccessPublic | QMC::MethodCloned, 0x80000000 | 8, {{
            { 0x80000000 | 10, 11 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    QtMocHelpers::UintData qt_constructors {};
    QtMocHelpers::ClassInfos qt_classinfo({
            {    1,    2 },
            {    3,    4 },
            {    5,    6 },
    });
    return QtMocHelpers::metaObjectData<QtQmlRemoteObjects, void>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums, qt_constructors, qt_classinfo);
}
Q_CONSTINIT const QMetaObject QtQmlRemoteObjects::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN18QtQmlRemoteObjectsE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN18QtQmlRemoteObjectsE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN18QtQmlRemoteObjectsE_t>.metaTypes,
    nullptr
} };

void QtQmlRemoteObjects::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<QtQmlRemoteObjects *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: { QJSValue _r = _t->watch((*reinterpret_cast<std::add_pointer_t<QRemoteObjectPendingCall>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2])));
            if (_a[0]) *reinterpret_cast<QJSValue*>(_a[0]) = std::move(_r); }  break;
        case 1: { QJSValue _r = _t->watch((*reinterpret_cast<std::add_pointer_t<QRemoteObjectPendingCall>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QJSValue*>(_a[0]) = std::move(_r); }  break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 0:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QRemoteObjectPendingCall >(); break;
            }
            break;
        case 1:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QRemoteObjectPendingCall >(); break;
            }
            break;
        }
    }
}

const QMetaObject *QtQmlRemoteObjects::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QtQmlRemoteObjects::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN18QtQmlRemoteObjectsE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int QtQmlRemoteObjects::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 2)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 2;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 2)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 2;
    }
    return _id;
}
namespace {
struct qt_meta_tag_ZN24QRemoteObjectNodeForeignE_t {};
} // unnamed namespace

template <> constexpr inline auto QRemoteObjectNodeForeign::qt_create_metaobjectdata<qt_meta_tag_ZN24QRemoteObjectNodeForeignE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "QRemoteObjectNodeForeign",
        "QML.Foreign",
        "QRemoteObjectNode",
        "QML.Element",
        "Node",
        "QML.AddedInVersion",
        "1292"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    QtMocHelpers::UintData qt_constructors {};
    QtMocHelpers::ClassInfos qt_classinfo({
            {    1,    2 },
            {    3,    4 },
            {    5,    6 },
    });
    return QtMocHelpers::metaObjectData<QRemoteObjectNodeForeign, void>(QMC::PropertyAccessInStaticMetaCall, qt_stringData,
            qt_methods, qt_properties, qt_enums, qt_constructors, qt_classinfo);
}
Q_CONSTINIT const QMetaObject QRemoteObjectNodeForeign::staticMetaObject = { {
    nullptr,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN24QRemoteObjectNodeForeignE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN24QRemoteObjectNodeForeignE_t>.data,
    nullptr,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN24QRemoteObjectNodeForeignE_t>.metaTypes,
    nullptr
} };

namespace {
struct qt_meta_tag_ZN33QRemoteObjectSettingsStoreForeignE_t {};
} // unnamed namespace

template <> constexpr inline auto QRemoteObjectSettingsStoreForeign::qt_create_metaobjectdata<qt_meta_tag_ZN33QRemoteObjectSettingsStoreForeignE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "QRemoteObjectSettingsStoreForeign",
        "QML.Foreign",
        "QRemoteObjectSettingsStore",
        "QML.Element",
        "SettingsStore",
        "QML.AddedInVersion",
        "1292"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    QtMocHelpers::UintData qt_constructors {};
    QtMocHelpers::ClassInfos qt_classinfo({
            {    1,    2 },
            {    3,    4 },
            {    5,    6 },
    });
    return QtMocHelpers::metaObjectData<QRemoteObjectSettingsStoreForeign, void>(QMC::PropertyAccessInStaticMetaCall, qt_stringData,
            qt_methods, qt_properties, qt_enums, qt_constructors, qt_classinfo);
}
Q_CONSTINIT const QMetaObject QRemoteObjectSettingsStoreForeign::staticMetaObject = { {
    nullptr,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN33QRemoteObjectSettingsStoreForeignE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN33QRemoteObjectSettingsStoreForeignE_t>.data,
    nullptr,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN33QRemoteObjectSettingsStoreForeignE_t>.metaTypes,
    nullptr
} };

namespace {
struct qt_meta_tag_ZN24QRemoteObjectHostForeignE_t {};
} // unnamed namespace

template <> constexpr inline auto QRemoteObjectHostForeign::qt_create_metaobjectdata<qt_meta_tag_ZN24QRemoteObjectHostForeignE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "QRemoteObjectHostForeign",
        "QML.Foreign",
        "QRemoteObjectHost",
        "QML.Element",
        "Host",
        "QML.AddedInVersion",
        "1295"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    QtMocHelpers::UintData qt_constructors {};
    QtMocHelpers::ClassInfos qt_classinfo({
            {    1,    2 },
            {    3,    4 },
            {    5,    6 },
    });
    return QtMocHelpers::metaObjectData<QRemoteObjectHostForeign, void>(QMC::PropertyAccessInStaticMetaCall, qt_stringData,
            qt_methods, qt_properties, qt_enums, qt_constructors, qt_classinfo);
}
Q_CONSTINIT const QMetaObject QRemoteObjectHostForeign::staticMetaObject = { {
    nullptr,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN24QRemoteObjectHostForeignE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN24QRemoteObjectHostForeignE_t>.data,
    nullptr,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN24QRemoteObjectHostForeignE_t>.metaTypes,
    nullptr
} };

namespace {
struct qt_meta_tag_ZN42QRemoteObjectAbstractPersistedStoreForeignE_t {};
} // unnamed namespace

template <> constexpr inline auto QRemoteObjectAbstractPersistedStoreForeign::qt_create_metaobjectdata<qt_meta_tag_ZN42QRemoteObjectAbstractPersistedStoreForeignE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "QRemoteObjectAbstractPersistedStoreForeign",
        "QML.Foreign",
        "QRemoteObjectAbstractPersistedStore",
        "QML.Element",
        "PersistedStore",
        "QML.Creatable",
        "false",
        "QML.UncreatableReason",
        "QRemoteObjectAbstractPersistedStore is Abstract",
        "QML.AddedInVersion",
        "1292"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    QtMocHelpers::UintData qt_constructors {};
    QtMocHelpers::ClassInfos qt_classinfo({
            {    1,    2 },
            {    3,    4 },
            {    5,    6 },
            {    7,    8 },
            {    9,   10 },
    });
    return QtMocHelpers::metaObjectData<QRemoteObjectAbstractPersistedStoreForeign, void>(QMC::PropertyAccessInStaticMetaCall, qt_stringData,
            qt_methods, qt_properties, qt_enums, qt_constructors, qt_classinfo);
}
Q_CONSTINIT const QMetaObject QRemoteObjectAbstractPersistedStoreForeign::staticMetaObject = { {
    nullptr,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN42QRemoteObjectAbstractPersistedStoreForeignE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN42QRemoteObjectAbstractPersistedStoreForeignE_t>.data,
    nullptr,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN42QRemoteObjectAbstractPersistedStoreForeignE_t>.metaTypes,
    nullptr
} };

QT_WARNING_POP
