/****************************************************************************
** Generated QML type registration code
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <QtQml/qqml.h>
#include <QtQml/qqmlmoduleregistration.h>

#if __has_include(<private/qremoteobjectsqml_p.h>)
#  include <private/qremoteobjectsqml_p.h>
#endif


#if !defined(QT_STATIC)
#define Q_QMLTYPE_EXPORT Q_DECL_EXPORT
#else
#define Q_QMLTYPE_EXPORT
#endif
Q_QMLTYPE_EXPORT void qml_register_types_QtRemoteObjects()
{
    qmlRegisterModule("QtRemoteObjects", 6, 0);
    QT_WARNING_PUSH QT_WARNING_DISABLE_DEPRECATED
    qmlRegisterTypesAndRevisions<QRemoteObjectAbstractPersistedStoreForeign>("QtRemoteObjects", 6);
    QMetaType::fromType<QRemoteObjectHostBase *>().id();
    qmlRegisterEnum<QRemoteObjectHostBase::AllowedSchemas>("QRemoteObjectHostBase::AllowedSchemas");
    qmlRegisterTypesAndRevisions<QRemoteObjectHostForeign>("QtRemoteObjects", 6);
    qmlRegisterTypesAndRevisions<QRemoteObjectNodeForeign>("QtRemoteObjects", 6);
    qmlRegisterEnum<QRemoteObjectNode::ErrorCode>("QRemoteObjectNode::ErrorCode");
    qmlRegisterTypesAndRevisions<QRemoteObjectSettingsStoreForeign>("QtRemoteObjects", 6);
    qmlRegisterTypesAndRevisions<QtQmlRemoteObjects>("QtRemoteObjects", 6);
    QT_WARNING_POP
    qmlRegisterModule("QtRemoteObjects", 6, 11);
}

static const QQmlModuleRegistration qtRemoteObjectsRegistration("QtRemoteObjects", qml_register_types_QtRemoteObjects);
