/****************************************************************************
** Meta object code from reading C++ file 'qremoteobjectabstractitemmodelreplica_p.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.2)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../qtremoteobjects/src/remoteobjects/qremoteobjectabstractitemmodelreplica_p.h"
#include <QtRemoteObjects/qremoteobjectnode.h>
#include <QtCore/qmetatype.h>
#include <QtCore/QList>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'qremoteobjectabstractitemmodelreplica_p.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.2. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN11SizeWatcherE_t {};
} // unnamed namespace

template <> constexpr inline auto SizeWatcher::qt_create_metaobjectdata<qt_meta_tag_ZN11SizeWatcherE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "SizeWatcher"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<SizeWatcher, qt_meta_tag_ZN11SizeWatcherE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject SizeWatcher::staticMetaObject = { {
    QMetaObject::SuperData::link<QRemoteObjectPendingCallWatcher::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN11SizeWatcherE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN11SizeWatcherE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN11SizeWatcherE_t>.metaTypes,
    nullptr
} };

void SizeWatcher::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<SizeWatcher *>(_o);
    (void)_t;
    (void)_c;
    (void)_id;
    (void)_a;
}

const QMetaObject *SizeWatcher::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *SizeWatcher::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN11SizeWatcherE_t>.strings))
        return static_cast<void*>(this);
    return QRemoteObjectPendingCallWatcher::qt_metacast(_clname);
}

int SizeWatcher::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QRemoteObjectPendingCallWatcher::qt_metacall(_c, _id, _a);
    return _id;
}
namespace {
struct qt_meta_tag_ZN10RowWatcherE_t {};
} // unnamed namespace

template <> constexpr inline auto RowWatcher::qt_create_metaobjectdata<qt_meta_tag_ZN10RowWatcherE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "RowWatcher"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<RowWatcher, qt_meta_tag_ZN10RowWatcherE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject RowWatcher::staticMetaObject = { {
    QMetaObject::SuperData::link<QRemoteObjectPendingCallWatcher::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10RowWatcherE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10RowWatcherE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN10RowWatcherE_t>.metaTypes,
    nullptr
} };

void RowWatcher::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<RowWatcher *>(_o);
    (void)_t;
    (void)_c;
    (void)_id;
    (void)_a;
}

const QMetaObject *RowWatcher::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *RowWatcher::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10RowWatcherE_t>.strings))
        return static_cast<void*>(this);
    return QRemoteObjectPendingCallWatcher::qt_metacast(_clname);
}

int RowWatcher::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QRemoteObjectPendingCallWatcher::qt_metacall(_c, _id, _a);
    return _id;
}
namespace {
struct qt_meta_tag_ZN13HeaderWatcherE_t {};
} // unnamed namespace

template <> constexpr inline auto HeaderWatcher::qt_create_metaobjectdata<qt_meta_tag_ZN13HeaderWatcherE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "HeaderWatcher"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<HeaderWatcher, qt_meta_tag_ZN13HeaderWatcherE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject HeaderWatcher::staticMetaObject = { {
    QMetaObject::SuperData::link<QRemoteObjectPendingCallWatcher::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN13HeaderWatcherE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN13HeaderWatcherE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN13HeaderWatcherE_t>.metaTypes,
    nullptr
} };

void HeaderWatcher::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<HeaderWatcher *>(_o);
    (void)_t;
    (void)_c;
    (void)_id;
    (void)_a;
}

const QMetaObject *HeaderWatcher::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *HeaderWatcher::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN13HeaderWatcherE_t>.strings))
        return static_cast<void*>(this);
    return QRemoteObjectPendingCallWatcher::qt_metacast(_clname);
}

int HeaderWatcher::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QRemoteObjectPendingCallWatcher::qt_metacall(_c, _id, _a);
    return _id;
}
namespace {
struct qt_meta_tag_ZN39QAbstractItemModelReplicaImplementationE_t {};
} // unnamed namespace

template <> constexpr inline auto QAbstractItemModelReplicaImplementation::qt_create_metaobjectdata<qt_meta_tag_ZN39QAbstractItemModelReplicaImplementationE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "QAbstractItemModelReplicaImplementation",
        "RemoteObject Type",
        "ServerModelAdapter",
        "availableRolesChanged",
        "",
        "dataChanged",
        "QtPrivate::IndexList",
        "topLeft",
        "bottomRight",
        "QList<int>",
        "roles",
        "rowsInserted",
        "parent",
        "first",
        "last",
        "rowsRemoved",
        "rowsMoved",
        "start",
        "end",
        "destination",
        "row",
        "currentChanged",
        "current",
        "previous",
        "modelReset",
        "headerDataChanged",
        "Qt::Orientation",
        "columnsInserted",
        "layoutChanged",
        "parents",
        "QAbstractItemModel::LayoutChangeHint",
        "hint",
        "replicaSizeRequest",
        "QRemoteObjectPendingReply<QSize>",
        "parentList",
        "replicaRowRequest",
        "QRemoteObjectPendingReply<QtPrivate::DataEntries>",
        "replicaHeaderRequest",
        "QRemoteObjectPendingReply<QVariantList>",
        "QList<Qt::Orientation>",
        "orientations",
        "sections",
        "replicaSetCurrentIndex",
        "index",
        "QItemSelectionModel::SelectionFlags",
        "command",
        "replicaSetData",
        "QVariant",
        "value",
        "role",
        "replicaCacheRequest",
        "QRemoteObjectPendingReply<QtPrivate::MetaAndDataEntries>",
        "size_t",
        "size",
        "onHeaderDataChanged",
        "orientation",
        "onDataChanged",
        "onRowsInserted",
        "onRowsRemoved",
        "onColumnsInserted",
        "onRowsMoved",
        "srcParent",
        "srcRow",
        "count",
        "destParent",
        "destRow",
        "onCurrentChanged",
        "onModelReset",
        "requestedData",
        "QRemoteObjectPendingCallWatcher*",
        "requestedHeaderData",
        "init",
        "fetchPendingData",
        "fetchPendingHeaderData",
        "handleInitDone",
        "watcher",
        "handleModelResetDone",
        "handleSizeDone",
        "onReplicaCurrentChanged",
        "QModelIndex",
        "fillCache",
        "QtPrivate::IndexValuePair",
        "pair",
        "onLayoutChanged",
        "availableRoles",
        "roleNames",
        "QIntHash"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'availableRolesChanged'
        QtMocHelpers::SignalData<void()>(3, 4, QMC::AccessPublic, QMetaType::Void),
        // Signal 'dataChanged'
        QtMocHelpers::SignalData<void(QtPrivate::IndexList, QtPrivate::IndexList, QList<int>)>(5, 4, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 6, 7 }, { 0x80000000 | 6, 8 }, { 0x80000000 | 9, 10 },
        }}),
        // Signal 'rowsInserted'
        QtMocHelpers::SignalData<void(QtPrivate::IndexList, int, int)>(11, 4, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 6, 12 }, { QMetaType::Int, 13 }, { QMetaType::Int, 14 },
        }}),
        // Signal 'rowsRemoved'
        QtMocHelpers::SignalData<void(QtPrivate::IndexList, int, int)>(15, 4, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 6, 12 }, { QMetaType::Int, 13 }, { QMetaType::Int, 14 },
        }}),
        // Signal 'rowsMoved'
        QtMocHelpers::SignalData<void(QtPrivate::IndexList, int, int, QtPrivate::IndexList, int)>(16, 4, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 6, 12 }, { QMetaType::Int, 17 }, { QMetaType::Int, 18 }, { 0x80000000 | 6, 19 },
            { QMetaType::Int, 20 },
        }}),
        // Signal 'currentChanged'
        QtMocHelpers::SignalData<void(QtPrivate::IndexList, QtPrivate::IndexList)>(21, 4, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 6, 22 }, { 0x80000000 | 6, 23 },
        }}),
        // Signal 'modelReset'
        QtMocHelpers::SignalData<void()>(24, 4, QMC::AccessPublic, QMetaType::Void),
        // Signal 'headerDataChanged'
        QtMocHelpers::SignalData<void(Qt::Orientation, int, int)>(25, 4, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 26, 4 }, { QMetaType::Int, 4 }, { QMetaType::Int, 4 },
        }}),
        // Signal 'columnsInserted'
        QtMocHelpers::SignalData<void(QtPrivate::IndexList, int, int)>(27, 4, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 6, 12 }, { QMetaType::Int, 13 }, { QMetaType::Int, 14 },
        }}),
        // Signal 'layoutChanged'
        QtMocHelpers::SignalData<void(QtPrivate::IndexList, QAbstractItemModel::LayoutChangeHint)>(28, 4, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 6, 29 }, { 0x80000000 | 30, 31 },
        }}),
        // Slot 'replicaSizeRequest'
        QtMocHelpers::SlotData<QRemoteObjectPendingReply<QSize>(QtPrivate::IndexList)>(32, 4, QMC::AccessPublic, 0x80000000 | 33, {{
            { 0x80000000 | 6, 34 },
        }}),
        // Slot 'replicaRowRequest'
        QtMocHelpers::SlotData<QRemoteObjectPendingReply<QtPrivate::DataEntries>(QtPrivate::IndexList, QtPrivate::IndexList, QList<int>)>(35, 4, QMC::AccessPublic, 0x80000000 | 36, {{
            { 0x80000000 | 6, 17 }, { 0x80000000 | 6, 18 }, { 0x80000000 | 9, 10 },
        }}),
        // Slot 'replicaHeaderRequest'
        QtMocHelpers::SlotData<QRemoteObjectPendingReply<QVariantList>(QList<Qt::Orientation>, QList<int>, QList<int>)>(37, 4, QMC::AccessPublic, 0x80000000 | 38, {{
            { 0x80000000 | 39, 40 }, { 0x80000000 | 9, 41 }, { 0x80000000 | 9, 10 },
        }}),
        // Slot 'replicaSetCurrentIndex'
        QtMocHelpers::SlotData<void(QtPrivate::IndexList, QItemSelectionModel::SelectionFlags)>(42, 4, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 6, 43 }, { 0x80000000 | 44, 45 },
        }}),
        // Slot 'replicaSetData'
        QtMocHelpers::SlotData<void(QtPrivate::IndexList, const QVariant &, int)>(46, 4, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 6, 43 }, { 0x80000000 | 47, 48 }, { QMetaType::Int, 49 },
        }}),
        // Slot 'replicaCacheRequest'
        QtMocHelpers::SlotData<QRemoteObjectPendingReply<QtPrivate::MetaAndDataEntries>(size_t, QList<int>)>(50, 4, QMC::AccessPublic, 0x80000000 | 51, {{
            { 0x80000000 | 52, 53 }, { 0x80000000 | 9, 10 },
        }}),
        // Slot 'onHeaderDataChanged'
        QtMocHelpers::SlotData<void(Qt::Orientation, int, int)>(54, 4, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 26, 55 }, { QMetaType::Int, 13 }, { QMetaType::Int, 14 },
        }}),
        // Slot 'onDataChanged'
        QtMocHelpers::SlotData<void(const QtPrivate::IndexList &, const QtPrivate::IndexList &, const QList<int> &)>(56, 4, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 6, 17 }, { 0x80000000 | 6, 18 }, { 0x80000000 | 9, 10 },
        }}),
        // Slot 'onRowsInserted'
        QtMocHelpers::SlotData<void(const QtPrivate::IndexList &, int, int)>(57, 4, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 6, 12 }, { QMetaType::Int, 17 }, { QMetaType::Int, 18 },
        }}),
        // Slot 'onRowsRemoved'
        QtMocHelpers::SlotData<void(const QtPrivate::IndexList &, int, int)>(58, 4, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 6, 12 }, { QMetaType::Int, 17 }, { QMetaType::Int, 18 },
        }}),
        // Slot 'onColumnsInserted'
        QtMocHelpers::SlotData<void(const QtPrivate::IndexList &, int, int)>(59, 4, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 6, 12 }, { QMetaType::Int, 17 }, { QMetaType::Int, 18 },
        }}),
        // Slot 'onRowsMoved'
        QtMocHelpers::SlotData<void(QtPrivate::IndexList, int, int, QtPrivate::IndexList, int)>(60, 4, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 6, 61 }, { QMetaType::Int, 62 }, { QMetaType::Int, 63 }, { 0x80000000 | 6, 64 },
            { QMetaType::Int, 65 },
        }}),
        // Slot 'onCurrentChanged'
        QtMocHelpers::SlotData<void(QtPrivate::IndexList, QtPrivate::IndexList)>(66, 4, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 6, 22 }, { 0x80000000 | 6, 23 },
        }}),
        // Slot 'onModelReset'
        QtMocHelpers::SlotData<void()>(67, 4, QMC::AccessPublic, QMetaType::Void),
        // Slot 'requestedData'
        QtMocHelpers::SlotData<void(QRemoteObjectPendingCallWatcher *)>(68, 4, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 69, 4 },
        }}),
        // Slot 'requestedHeaderData'
        QtMocHelpers::SlotData<void(QRemoteObjectPendingCallWatcher *)>(70, 4, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 69, 4 },
        }}),
        // Slot 'init'
        QtMocHelpers::SlotData<void()>(71, 4, QMC::AccessPublic, QMetaType::Void),
        // Slot 'fetchPendingData'
        QtMocHelpers::SlotData<void()>(72, 4, QMC::AccessPublic, QMetaType::Void),
        // Slot 'fetchPendingHeaderData'
        QtMocHelpers::SlotData<void()>(73, 4, QMC::AccessPublic, QMetaType::Void),
        // Slot 'handleInitDone'
        QtMocHelpers::SlotData<void(QRemoteObjectPendingCallWatcher *)>(74, 4, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 69, 75 },
        }}),
        // Slot 'handleModelResetDone'
        QtMocHelpers::SlotData<void(QRemoteObjectPendingCallWatcher *)>(76, 4, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 69, 75 },
        }}),
        // Slot 'handleSizeDone'
        QtMocHelpers::SlotData<void(QRemoteObjectPendingCallWatcher *)>(77, 4, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 69, 75 },
        }}),
        // Slot 'onReplicaCurrentChanged'
        QtMocHelpers::SlotData<void(const QModelIndex &, const QModelIndex &)>(78, 4, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 79, 22 }, { 0x80000000 | 79, 23 },
        }}),
        // Slot 'fillCache'
        QtMocHelpers::SlotData<void(const QtPrivate::IndexValuePair &, const QList<int> &)>(80, 4, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 81, 82 }, { 0x80000000 | 9, 10 },
        }}),
        // Slot 'onLayoutChanged'
        QtMocHelpers::SlotData<void(const QtPrivate::IndexList &, QAbstractItemModel::LayoutChangeHint)>(83, 4, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 6, 29 }, { 0x80000000 | 30, 31 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'availableRoles'
        QtMocHelpers::PropertyData<QList<int>>(84, 0x80000000 | 9, QMC::DefaultPropertyFlags | QMC::EnumOrFlag, 0),
        // property 'roleNames'
        QtMocHelpers::PropertyData<QIntHash>(85, 0x80000000 | 86, QMC::DefaultPropertyFlags | QMC::EnumOrFlag),
    };
    QtMocHelpers::UintData qt_enums {
    };
    QtMocHelpers::UintData qt_constructors {};
    QtMocHelpers::ClassInfos qt_classinfo({
            {    1,    2 },
    });
    return QtMocHelpers::metaObjectData<QAbstractItemModelReplicaImplementation, qt_meta_tag_ZN39QAbstractItemModelReplicaImplementationE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums, qt_constructors, qt_classinfo);
}
Q_CONSTINIT const QMetaObject QAbstractItemModelReplicaImplementation::staticMetaObject = { {
    QMetaObject::SuperData::link<QRemoteObjectReplica::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN39QAbstractItemModelReplicaImplementationE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN39QAbstractItemModelReplicaImplementationE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN39QAbstractItemModelReplicaImplementationE_t>.metaTypes,
    nullptr
} };

void QAbstractItemModelReplicaImplementation::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<QAbstractItemModelReplicaImplementation *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->availableRolesChanged(); break;
        case 1: _t->dataChanged((*reinterpret_cast<std::add_pointer_t<QtPrivate::IndexList>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QtPrivate::IndexList>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QList<int>>>(_a[3]))); break;
        case 2: _t->rowsInserted((*reinterpret_cast<std::add_pointer_t<QtPrivate::IndexList>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[3]))); break;
        case 3: _t->rowsRemoved((*reinterpret_cast<std::add_pointer_t<QtPrivate::IndexList>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[3]))); break;
        case 4: _t->rowsMoved((*reinterpret_cast<std::add_pointer_t<QtPrivate::IndexList>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<QtPrivate::IndexList>>(_a[4])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[5]))); break;
        case 5: _t->currentChanged((*reinterpret_cast<std::add_pointer_t<QtPrivate::IndexList>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QtPrivate::IndexList>>(_a[2]))); break;
        case 6: _t->modelReset(); break;
        case 7: _t->headerDataChanged((*reinterpret_cast<std::add_pointer_t<Qt::Orientation>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[3]))); break;
        case 8: _t->columnsInserted((*reinterpret_cast<std::add_pointer_t<QtPrivate::IndexList>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[3]))); break;
        case 9: _t->layoutChanged((*reinterpret_cast<std::add_pointer_t<QtPrivate::IndexList>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QAbstractItemModel::LayoutChangeHint>>(_a[2]))); break;
        case 10: { QRemoteObjectPendingReply<QSize> _r = _t->replicaSizeRequest((*reinterpret_cast<std::add_pointer_t<QtPrivate::IndexList>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QRemoteObjectPendingReply<QSize>*>(_a[0]) = std::move(_r); }  break;
        case 11: { QRemoteObjectPendingReply<QtPrivate::DataEntries> _r = _t->replicaRowRequest((*reinterpret_cast<std::add_pointer_t<QtPrivate::IndexList>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QtPrivate::IndexList>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QList<int>>>(_a[3])));
            if (_a[0]) *reinterpret_cast<QRemoteObjectPendingReply<QtPrivate::DataEntries>*>(_a[0]) = std::move(_r); }  break;
        case 12: { QRemoteObjectPendingReply<QVariantList> _r = _t->replicaHeaderRequest((*reinterpret_cast<std::add_pointer_t<QList<Qt::Orientation>>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QList<int>>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QList<int>>>(_a[3])));
            if (_a[0]) *reinterpret_cast<QRemoteObjectPendingReply<QVariantList>*>(_a[0]) = std::move(_r); }  break;
        case 13: _t->replicaSetCurrentIndex((*reinterpret_cast<std::add_pointer_t<QtPrivate::IndexList>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QItemSelectionModel::SelectionFlags>>(_a[2]))); break;
        case 14: _t->replicaSetData((*reinterpret_cast<std::add_pointer_t<QtPrivate::IndexList>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QVariant>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[3]))); break;
        case 15: { QRemoteObjectPendingReply<QtPrivate::MetaAndDataEntries> _r = _t->replicaCacheRequest((*reinterpret_cast<std::add_pointer_t<size_t>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QList<int>>>(_a[2])));
            if (_a[0]) *reinterpret_cast<QRemoteObjectPendingReply<QtPrivate::MetaAndDataEntries>*>(_a[0]) = std::move(_r); }  break;
        case 16: _t->onHeaderDataChanged((*reinterpret_cast<std::add_pointer_t<Qt::Orientation>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[3]))); break;
        case 17: _t->onDataChanged((*reinterpret_cast<std::add_pointer_t<QtPrivate::IndexList>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QtPrivate::IndexList>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QList<int>>>(_a[3]))); break;
        case 18: _t->onRowsInserted((*reinterpret_cast<std::add_pointer_t<QtPrivate::IndexList>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[3]))); break;
        case 19: _t->onRowsRemoved((*reinterpret_cast<std::add_pointer_t<QtPrivate::IndexList>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[3]))); break;
        case 20: _t->onColumnsInserted((*reinterpret_cast<std::add_pointer_t<QtPrivate::IndexList>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[3]))); break;
        case 21: _t->onRowsMoved((*reinterpret_cast<std::add_pointer_t<QtPrivate::IndexList>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<QtPrivate::IndexList>>(_a[4])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[5]))); break;
        case 22: _t->onCurrentChanged((*reinterpret_cast<std::add_pointer_t<QtPrivate::IndexList>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QtPrivate::IndexList>>(_a[2]))); break;
        case 23: _t->onModelReset(); break;
        case 24: _t->requestedData((*reinterpret_cast<std::add_pointer_t<QRemoteObjectPendingCallWatcher*>>(_a[1]))); break;
        case 25: _t->requestedHeaderData((*reinterpret_cast<std::add_pointer_t<QRemoteObjectPendingCallWatcher*>>(_a[1]))); break;
        case 26: _t->init(); break;
        case 27: _t->fetchPendingData(); break;
        case 28: _t->fetchPendingHeaderData(); break;
        case 29: _t->handleInitDone((*reinterpret_cast<std::add_pointer_t<QRemoteObjectPendingCallWatcher*>>(_a[1]))); break;
        case 30: _t->handleModelResetDone((*reinterpret_cast<std::add_pointer_t<QRemoteObjectPendingCallWatcher*>>(_a[1]))); break;
        case 31: _t->handleSizeDone((*reinterpret_cast<std::add_pointer_t<QRemoteObjectPendingCallWatcher*>>(_a[1]))); break;
        case 32: _t->onReplicaCurrentChanged((*reinterpret_cast<std::add_pointer_t<QModelIndex>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QModelIndex>>(_a[2]))); break;
        case 33: _t->fillCache((*reinterpret_cast<std::add_pointer_t<QtPrivate::IndexValuePair>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QList<int>>>(_a[2]))); break;
        case 34: _t->onLayoutChanged((*reinterpret_cast<std::add_pointer_t<QtPrivate::IndexList>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QAbstractItemModel::LayoutChangeHint>>(_a[2]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 1:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 2:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QList<int> >(); break;
            case 1:
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QtPrivate::IndexList >(); break;
            }
            break;
        case 2:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QtPrivate::IndexList >(); break;
            }
            break;
        case 3:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QtPrivate::IndexList >(); break;
            }
            break;
        case 4:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 3:
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QtPrivate::IndexList >(); break;
            }
            break;
        case 5:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 1:
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QtPrivate::IndexList >(); break;
            }
            break;
        case 7:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< Qt::Orientation >(); break;
            }
            break;
        case 8:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QtPrivate::IndexList >(); break;
            }
            break;
        case 9:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QtPrivate::IndexList >(); break;
            }
            break;
        case 10:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QtPrivate::IndexList >(); break;
            }
            break;
        case 11:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 2:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QList<int> >(); break;
            case 1:
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QtPrivate::IndexList >(); break;
            }
            break;
        case 12:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QList<Qt::Orientation> >(); break;
            case 2:
            case 1:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QList<int> >(); break;
            }
            break;
        case 13:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 1:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QItemSelectionModel::SelectionFlags >(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QtPrivate::IndexList >(); break;
            }
            break;
        case 14:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QtPrivate::IndexList >(); break;
            }
            break;
        case 15:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 1:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QList<int> >(); break;
            }
            break;
        case 16:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< Qt::Orientation >(); break;
            }
            break;
        case 17:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 2:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QList<int> >(); break;
            case 1:
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QtPrivate::IndexList >(); break;
            }
            break;
        case 18:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QtPrivate::IndexList >(); break;
            }
            break;
        case 19:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QtPrivate::IndexList >(); break;
            }
            break;
        case 20:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QtPrivate::IndexList >(); break;
            }
            break;
        case 21:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 3:
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QtPrivate::IndexList >(); break;
            }
            break;
        case 22:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 1:
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QtPrivate::IndexList >(); break;
            }
            break;
        case 24:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QRemoteObjectPendingCallWatcher* >(); break;
            }
            break;
        case 25:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QRemoteObjectPendingCallWatcher* >(); break;
            }
            break;
        case 29:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QRemoteObjectPendingCallWatcher* >(); break;
            }
            break;
        case 30:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QRemoteObjectPendingCallWatcher* >(); break;
            }
            break;
        case 31:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QRemoteObjectPendingCallWatcher* >(); break;
            }
            break;
        case 33:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 1:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QList<int> >(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QtPrivate::IndexValuePair >(); break;
            }
            break;
        case 34:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QtPrivate::IndexList >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (QAbstractItemModelReplicaImplementation::*)()>(_a, &QAbstractItemModelReplicaImplementation::availableRolesChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (QAbstractItemModelReplicaImplementation::*)(QtPrivate::IndexList , QtPrivate::IndexList , QList<int> )>(_a, &QAbstractItemModelReplicaImplementation::dataChanged, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (QAbstractItemModelReplicaImplementation::*)(QtPrivate::IndexList , int , int )>(_a, &QAbstractItemModelReplicaImplementation::rowsInserted, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (QAbstractItemModelReplicaImplementation::*)(QtPrivate::IndexList , int , int )>(_a, &QAbstractItemModelReplicaImplementation::rowsRemoved, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (QAbstractItemModelReplicaImplementation::*)(QtPrivate::IndexList , int , int , QtPrivate::IndexList , int )>(_a, &QAbstractItemModelReplicaImplementation::rowsMoved, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (QAbstractItemModelReplicaImplementation::*)(QtPrivate::IndexList , QtPrivate::IndexList )>(_a, &QAbstractItemModelReplicaImplementation::currentChanged, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (QAbstractItemModelReplicaImplementation::*)()>(_a, &QAbstractItemModelReplicaImplementation::modelReset, 6))
            return;
        if (QtMocHelpers::indexOfMethod<void (QAbstractItemModelReplicaImplementation::*)(Qt::Orientation , int , int )>(_a, &QAbstractItemModelReplicaImplementation::headerDataChanged, 7))
            return;
        if (QtMocHelpers::indexOfMethod<void (QAbstractItemModelReplicaImplementation::*)(QtPrivate::IndexList , int , int )>(_a, &QAbstractItemModelReplicaImplementation::columnsInserted, 8))
            return;
        if (QtMocHelpers::indexOfMethod<void (QAbstractItemModelReplicaImplementation::*)(QtPrivate::IndexList , QAbstractItemModel::LayoutChangeHint )>(_a, &QAbstractItemModelReplicaImplementation::layoutChanged, 9))
            return;
    }
    if (_c == QMetaObject::RegisterPropertyMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 1:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QIntHash >(); break;
        case 0:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QList<int> >(); break;
        }
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<QList<int>*>(_v) = _t->availableRoles(); break;
        case 1: *reinterpret_cast<QIntHash*>(_v) = _t->roleNames(); break;
        default: break;
        }
    }
}

const QMetaObject *QAbstractItemModelReplicaImplementation::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QAbstractItemModelReplicaImplementation::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN39QAbstractItemModelReplicaImplementationE_t>.strings))
        return static_cast<void*>(this);
    return QRemoteObjectReplica::qt_metacast(_clname);
}

int QAbstractItemModelReplicaImplementation::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QRemoteObjectReplica::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 35)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 35;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 35)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 35;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 2;
    }
    return _id;
}

// SIGNAL 0
void QAbstractItemModelReplicaImplementation::availableRolesChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void QAbstractItemModelReplicaImplementation::dataChanged(QtPrivate::IndexList _t1, QtPrivate::IndexList _t2, QList<int> _t3)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1, _t2, _t3);
}

// SIGNAL 2
void QAbstractItemModelReplicaImplementation::rowsInserted(QtPrivate::IndexList _t1, int _t2, int _t3)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1, _t2, _t3);
}

// SIGNAL 3
void QAbstractItemModelReplicaImplementation::rowsRemoved(QtPrivate::IndexList _t1, int _t2, int _t3)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1, _t2, _t3);
}

// SIGNAL 4
void QAbstractItemModelReplicaImplementation::rowsMoved(QtPrivate::IndexList _t1, int _t2, int _t3, QtPrivate::IndexList _t4, int _t5)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 4, nullptr, _t1, _t2, _t3, _t4, _t5);
}

// SIGNAL 5
void QAbstractItemModelReplicaImplementation::currentChanged(QtPrivate::IndexList _t1, QtPrivate::IndexList _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 5, nullptr, _t1, _t2);
}

// SIGNAL 6
void QAbstractItemModelReplicaImplementation::modelReset()
{
    QMetaObject::activate(this, &staticMetaObject, 6, nullptr);
}

// SIGNAL 7
void QAbstractItemModelReplicaImplementation::headerDataChanged(Qt::Orientation _t1, int _t2, int _t3)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 7, nullptr, _t1, _t2, _t3);
}

// SIGNAL 8
void QAbstractItemModelReplicaImplementation::columnsInserted(QtPrivate::IndexList _t1, int _t2, int _t3)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 8, nullptr, _t1, _t2, _t3);
}

// SIGNAL 9
void QAbstractItemModelReplicaImplementation::layoutChanged(QtPrivate::IndexList _t1, QAbstractItemModel::LayoutChangeHint _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 9, nullptr, _t1, _t2);
}
QT_WARNING_POP
