/****************************************************************************
** Meta object code from reading C++ file 'qremoteobjectabstractitemmodeladapter_p.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.2)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../qtremoteobjects/src/remoteobjects/qremoteobjectabstractitemmodeladapter_p.h"
#include <QtCore/qmetatype.h>
#include <QtCore/QList>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'qremoteobjectabstractitemmodeladapter_p.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.2. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN31QAbstractItemModelSourceAdapterE_t {};
} // unnamed namespace

template <> constexpr inline auto QAbstractItemModelSourceAdapter::qt_create_metaobjectdata<qt_meta_tag_ZN31QAbstractItemModelSourceAdapterE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "QAbstractItemModelSourceAdapter",
        "availableRolesChanged",
        "",
        "dataChanged",
        "QtPrivate::IndexList",
        "topLeft",
        "bottomRight",
        "QList<int>",
        "roles",
        "rowsInserted",
        "parent",
        "start",
        "end",
        "rowsRemoved",
        "rowsMoved",
        "sourceParent",
        "sourceRow",
        "count",
        "destinationParent",
        "destinationChild",
        "currentChanged",
        "current",
        "previous",
        "columnsInserted",
        "layoutChanged",
        "parents",
        "QAbstractItemModel::LayoutChangeHint",
        "hint",
        "availableRoles",
        "setAvailableRoles",
        "roleNames",
        "QIntHash",
        "replicaSizeRequest",
        "QSize",
        "parentList",
        "replicaRowRequest",
        "QtPrivate::DataEntries",
        "replicaHeaderRequest",
        "QVariantList",
        "QList<Qt::Orientation>",
        "orientations",
        "sections",
        "replicaSetCurrentIndex",
        "index",
        "QItemSelectionModel::SelectionFlags",
        "command",
        "replicaSetData",
        "QVariant",
        "value",
        "role",
        "replicaCacheRequest",
        "QtPrivate::MetaAndDataEntries",
        "size_t",
        "size",
        "sourceDataChanged",
        "QModelIndex",
        "sourceRowsInserted",
        "sourceColumnsInserted",
        "sourceRowsRemoved",
        "sourceRowsMoved",
        "sourceCurrentChanged",
        "sourceLayoutChanged",
        "QList<QPersistentModelIndex>",
        "QAbstractItemModel*",
        "object",
        "QItemSelectionModel*",
        "sel"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'availableRolesChanged'
        QtMocHelpers::SignalData<void()>(1, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'dataChanged'
        QtMocHelpers::SignalData<void(QtPrivate::IndexList, QtPrivate::IndexList, QList<int>) const>(3, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 4, 5 }, { 0x80000000 | 4, 6 }, { 0x80000000 | 7, 8 },
        }}),
        // Signal 'rowsInserted'
        QtMocHelpers::SignalData<void(QtPrivate::IndexList, int, int) const>(9, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 4, 10 }, { QMetaType::Int, 11 }, { QMetaType::Int, 12 },
        }}),
        // Signal 'rowsRemoved'
        QtMocHelpers::SignalData<void(QtPrivate::IndexList, int, int) const>(13, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 4, 10 }, { QMetaType::Int, 11 }, { QMetaType::Int, 12 },
        }}),
        // Signal 'rowsMoved'
        QtMocHelpers::SignalData<void(QtPrivate::IndexList, int, int, QtPrivate::IndexList, int) const>(14, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 4, 15 }, { QMetaType::Int, 16 }, { QMetaType::Int, 17 }, { 0x80000000 | 4, 18 },
            { QMetaType::Int, 19 },
        }}),
        // Signal 'currentChanged'
        QtMocHelpers::SignalData<void(QtPrivate::IndexList, QtPrivate::IndexList)>(20, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 4, 21 }, { 0x80000000 | 4, 22 },
        }}),
        // Signal 'columnsInserted'
        QtMocHelpers::SignalData<void(QtPrivate::IndexList, int, int) const>(23, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 4, 10 }, { QMetaType::Int, 11 }, { QMetaType::Int, 12 },
        }}),
        // Signal 'layoutChanged'
        QtMocHelpers::SignalData<void(QtPrivate::IndexList, QAbstractItemModel::LayoutChangeHint)>(24, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 4, 25 }, { 0x80000000 | 26, 27 },
        }}),
        // Slot 'availableRoles'
        QtMocHelpers::SlotData<QList<int>() const>(28, 2, QMC::AccessPublic, 0x80000000 | 7),
        // Slot 'setAvailableRoles'
        QtMocHelpers::SlotData<void(QList<int>)>(29, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 7, 28 },
        }}),
        // Slot 'roleNames'
        QtMocHelpers::SlotData<QIntHash() const>(30, 2, QMC::AccessPublic, 0x80000000 | 31),
        // Slot 'replicaSizeRequest'
        QtMocHelpers::SlotData<QSize(QtPrivate::IndexList)>(32, 2, QMC::AccessPublic, 0x80000000 | 33, {{
            { 0x80000000 | 4, 34 },
        }}),
        // Slot 'replicaRowRequest'
        QtMocHelpers::SlotData<QtPrivate::DataEntries(QtPrivate::IndexList, QtPrivate::IndexList, QList<int>)>(35, 2, QMC::AccessPublic, 0x80000000 | 36, {{
            { 0x80000000 | 4, 11 }, { 0x80000000 | 4, 12 }, { 0x80000000 | 7, 8 },
        }}),
        // Slot 'replicaHeaderRequest'
        QtMocHelpers::SlotData<QVariantList(QList<Qt::Orientation>, QList<int>, QList<int>)>(37, 2, QMC::AccessPublic, 0x80000000 | 38, {{
            { 0x80000000 | 39, 40 }, { 0x80000000 | 7, 41 }, { 0x80000000 | 7, 8 },
        }}),
        // Slot 'replicaSetCurrentIndex'
        QtMocHelpers::SlotData<void(QtPrivate::IndexList, QItemSelectionModel::SelectionFlags)>(42, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 4, 43 }, { 0x80000000 | 44, 45 },
        }}),
        // Slot 'replicaSetData'
        QtMocHelpers::SlotData<void(const QtPrivate::IndexList &, const QVariant &, int)>(46, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 4, 43 }, { 0x80000000 | 47, 48 }, { QMetaType::Int, 49 },
        }}),
        // Slot 'replicaCacheRequest'
        QtMocHelpers::SlotData<QtPrivate::MetaAndDataEntries(size_t, const QList<int> &)>(50, 2, QMC::AccessPublic, 0x80000000 | 51, {{
            { 0x80000000 | 52, 53 }, { 0x80000000 | 7, 8 },
        }}),
        // Slot 'sourceDataChanged'
        QtMocHelpers::SlotData<void(const QModelIndex &, const QModelIndex &, const QList<int> &) const>(54, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 55, 5 }, { 0x80000000 | 55, 6 }, { 0x80000000 | 7, 8 },
        }}),
        // Slot 'sourceDataChanged'
        QtMocHelpers::SlotData<void(const QModelIndex &, const QModelIndex &) const>(54, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void, {{
            { 0x80000000 | 55, 5 }, { 0x80000000 | 55, 6 },
        }}),
        // Slot 'sourceRowsInserted'
        QtMocHelpers::SlotData<void(const QModelIndex &, int, int)>(56, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 55, 10 }, { QMetaType::Int, 11 }, { QMetaType::Int, 12 },
        }}),
        // Slot 'sourceColumnsInserted'
        QtMocHelpers::SlotData<void(const QModelIndex &, int, int)>(57, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 55, 10 }, { QMetaType::Int, 11 }, { QMetaType::Int, 12 },
        }}),
        // Slot 'sourceRowsRemoved'
        QtMocHelpers::SlotData<void(const QModelIndex &, int, int)>(58, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 55, 10 }, { QMetaType::Int, 11 }, { QMetaType::Int, 12 },
        }}),
        // Slot 'sourceRowsMoved'
        QtMocHelpers::SlotData<void(const QModelIndex &, int, int, const QModelIndex &, int) const>(59, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 55, 15 }, { QMetaType::Int, 16 }, { QMetaType::Int, 17 }, { 0x80000000 | 55, 18 },
            { QMetaType::Int, 19 },
        }}),
        // Slot 'sourceCurrentChanged'
        QtMocHelpers::SlotData<void(const QModelIndex &, const QModelIndex &)>(60, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 55, 21 }, { 0x80000000 | 55, 22 },
        }}),
        // Slot 'sourceLayoutChanged'
        QtMocHelpers::SlotData<void(const QList<QPersistentModelIndex> &, QAbstractItemModel::LayoutChangeHint)>(61, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 62, 25 }, { 0x80000000 | 26, 27 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'availableRoles'
        QtMocHelpers::PropertyData<QList<int>>(28, 0x80000000 | 7, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 0),
        // property 'roleNames'
        QtMocHelpers::PropertyData<QIntHash>(30, 0x80000000 | 31, QMC::DefaultPropertyFlags | QMC::EnumOrFlag),
    };
    QtMocHelpers::UintData qt_enums {
    };
    using Constructor = QtMocHelpers::NoType;
    QtMocHelpers::UintData qt_constructors {
        QtMocHelpers::ConstructorData<Constructor(QAbstractItemModel *, QItemSelectionModel *, const QList<int> &)>(2, QMC::AccessPublic, {{
            { 0x80000000 | 63, 64 }, { 0x80000000 | 65, 66 }, { 0x80000000 | 7, 8 },
        }}),
        QtMocHelpers::ConstructorData<Constructor(QAbstractItemModel *, QItemSelectionModel *)>(2, QMC::AccessPublic | QMC::MethodCloned, {{
            { 0x80000000 | 63, 64 }, { 0x80000000 | 65, 66 },
        }}),
    };
    return QtMocHelpers::metaObjectData<QAbstractItemModelSourceAdapter, qt_meta_tag_ZN31QAbstractItemModelSourceAdapterE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums, qt_constructors);
}
Q_CONSTINIT const QMetaObject QAbstractItemModelSourceAdapter::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN31QAbstractItemModelSourceAdapterE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN31QAbstractItemModelSourceAdapterE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN31QAbstractItemModelSourceAdapterE_t>.metaTypes,
    nullptr
} };

void QAbstractItemModelSourceAdapter::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<QAbstractItemModelSourceAdapter *>(_o);
    if (_c == QMetaObject::CreateInstance) {
        switch (_id) {
        case 0: { QAbstractItemModelSourceAdapter *_r = new QAbstractItemModelSourceAdapter((*reinterpret_cast<std::add_pointer_t<QAbstractItemModel*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QItemSelectionModel*>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QList<int>>>(_a[3])));
            if (_a[0]) *reinterpret_cast<QObject**>(_a[0]) = _r; } break;
        case 1: { QAbstractItemModelSourceAdapter *_r = new QAbstractItemModelSourceAdapter((*reinterpret_cast<std::add_pointer_t<QAbstractItemModel*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QItemSelectionModel*>>(_a[2])));
            if (_a[0]) *reinterpret_cast<QObject**>(_a[0]) = _r; } break;
        default: break;
        }
    }
    if (_c == QMetaObject::ConstructInPlace) {
        switch (_id) {
        case 0: { new (_a[0]) QAbstractItemModelSourceAdapter((*reinterpret_cast<std::add_pointer_t<QAbstractItemModel*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QItemSelectionModel*>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QList<int>>>(_a[3]))); } break;
        case 1: { new (_a[0]) QAbstractItemModelSourceAdapter((*reinterpret_cast<std::add_pointer_t<QAbstractItemModel*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QItemSelectionModel*>>(_a[2]))); } break;
        default: break;
        }
    }
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->availableRolesChanged(); break;
        case 1: _t->dataChanged((*reinterpret_cast<std::add_pointer_t<QtPrivate::IndexList>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QtPrivate::IndexList>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QList<int>>>(_a[3]))); break;
        case 2: _t->rowsInserted((*reinterpret_cast<std::add_pointer_t<QtPrivate::IndexList>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[3]))); break;
        case 3: _t->rowsRemoved((*reinterpret_cast<std::add_pointer_t<QtPrivate::IndexList>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[3]))); break;
        case 4: _t->rowsMoved((*reinterpret_cast<std::add_pointer_t<QtPrivate::IndexList>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<QtPrivate::IndexList>>(_a[4])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[5]))); break;
        case 5: _t->currentChanged((*reinterpret_cast<std::add_pointer_t<QtPrivate::IndexList>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QtPrivate::IndexList>>(_a[2]))); break;
        case 6: _t->columnsInserted((*reinterpret_cast<std::add_pointer_t<QtPrivate::IndexList>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[3]))); break;
        case 7: _t->layoutChanged((*reinterpret_cast<std::add_pointer_t<QtPrivate::IndexList>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QAbstractItemModel::LayoutChangeHint>>(_a[2]))); break;
        case 8: { QList<int> _r = _t->availableRoles();
            if (_a[0]) *reinterpret_cast<QList<int>*>(_a[0]) = std::move(_r); }  break;
        case 9: _t->setAvailableRoles((*reinterpret_cast<std::add_pointer_t<QList<int>>>(_a[1]))); break;
        case 10: { QIntHash _r = _t->roleNames();
            if (_a[0]) *reinterpret_cast<QIntHash*>(_a[0]) = std::move(_r); }  break;
        case 11: { QSize _r = _t->replicaSizeRequest((*reinterpret_cast<std::add_pointer_t<QtPrivate::IndexList>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QSize*>(_a[0]) = std::move(_r); }  break;
        case 12: { QtPrivate::DataEntries _r = _t->replicaRowRequest((*reinterpret_cast<std::add_pointer_t<QtPrivate::IndexList>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QtPrivate::IndexList>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QList<int>>>(_a[3])));
            if (_a[0]) *reinterpret_cast<QtPrivate::DataEntries*>(_a[0]) = std::move(_r); }  break;
        case 13: { QVariantList _r = _t->replicaHeaderRequest((*reinterpret_cast<std::add_pointer_t<QList<Qt::Orientation>>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QList<int>>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QList<int>>>(_a[3])));
            if (_a[0]) *reinterpret_cast<QVariantList*>(_a[0]) = std::move(_r); }  break;
        case 14: _t->replicaSetCurrentIndex((*reinterpret_cast<std::add_pointer_t<QtPrivate::IndexList>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QItemSelectionModel::SelectionFlags>>(_a[2]))); break;
        case 15: _t->replicaSetData((*reinterpret_cast<std::add_pointer_t<QtPrivate::IndexList>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QVariant>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[3]))); break;
        case 16: { QtPrivate::MetaAndDataEntries _r = _t->replicaCacheRequest((*reinterpret_cast<std::add_pointer_t<size_t>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QList<int>>>(_a[2])));
            if (_a[0]) *reinterpret_cast<QtPrivate::MetaAndDataEntries*>(_a[0]) = std::move(_r); }  break;
        case 17: _t->sourceDataChanged((*reinterpret_cast<std::add_pointer_t<QModelIndex>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QModelIndex>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QList<int>>>(_a[3]))); break;
        case 18: _t->sourceDataChanged((*reinterpret_cast<std::add_pointer_t<QModelIndex>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QModelIndex>>(_a[2]))); break;
        case 19: _t->sourceRowsInserted((*reinterpret_cast<std::add_pointer_t<QModelIndex>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[3]))); break;
        case 20: _t->sourceColumnsInserted((*reinterpret_cast<std::add_pointer_t<QModelIndex>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[3]))); break;
        case 21: _t->sourceRowsRemoved((*reinterpret_cast<std::add_pointer_t<QModelIndex>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[3]))); break;
        case 22: _t->sourceRowsMoved((*reinterpret_cast<std::add_pointer_t<QModelIndex>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<QModelIndex>>(_a[4])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[5]))); break;
        case 23: _t->sourceCurrentChanged((*reinterpret_cast<std::add_pointer_t<QModelIndex>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QModelIndex>>(_a[2]))); break;
        case 24: _t->sourceLayoutChanged((*reinterpret_cast<std::add_pointer_t<QList<QPersistentModelIndex>>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QAbstractItemModel::LayoutChangeHint>>(_a[2]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 1:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 2:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QList<int> >(); break;
            case 1:
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QtPrivate::IndexList >(); break;
            }
            break;
        case 2:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QtPrivate::IndexList >(); break;
            }
            break;
        case 3:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QtPrivate::IndexList >(); break;
            }
            break;
        case 4:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 3:
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QtPrivate::IndexList >(); break;
            }
            break;
        case 5:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 1:
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QtPrivate::IndexList >(); break;
            }
            break;
        case 6:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QtPrivate::IndexList >(); break;
            }
            break;
        case 7:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QtPrivate::IndexList >(); break;
            }
            break;
        case 9:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QList<int> >(); break;
            }
            break;
        case 11:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QtPrivate::IndexList >(); break;
            }
            break;
        case 12:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 2:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QList<int> >(); break;
            case 1:
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QtPrivate::IndexList >(); break;
            }
            break;
        case 13:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QList<Qt::Orientation> >(); break;
            case 2:
            case 1:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QList<int> >(); break;
            }
            break;
        case 14:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 1:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QItemSelectionModel::SelectionFlags >(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QtPrivate::IndexList >(); break;
            }
            break;
        case 15:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QtPrivate::IndexList >(); break;
            }
            break;
        case 16:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 1:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QList<int> >(); break;
            }
            break;
        case 17:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 2:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QList<int> >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (QAbstractItemModelSourceAdapter::*)()>(_a, &QAbstractItemModelSourceAdapter::availableRolesChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (QAbstractItemModelSourceAdapter::*)(QtPrivate::IndexList , QtPrivate::IndexList , QList<int> ) const>(_a, &QAbstractItemModelSourceAdapter::dataChanged, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (QAbstractItemModelSourceAdapter::*)(QtPrivate::IndexList , int , int ) const>(_a, &QAbstractItemModelSourceAdapter::rowsInserted, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (QAbstractItemModelSourceAdapter::*)(QtPrivate::IndexList , int , int ) const>(_a, &QAbstractItemModelSourceAdapter::rowsRemoved, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (QAbstractItemModelSourceAdapter::*)(QtPrivate::IndexList , int , int , QtPrivate::IndexList , int ) const>(_a, &QAbstractItemModelSourceAdapter::rowsMoved, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (QAbstractItemModelSourceAdapter::*)(QtPrivate::IndexList , QtPrivate::IndexList )>(_a, &QAbstractItemModelSourceAdapter::currentChanged, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (QAbstractItemModelSourceAdapter::*)(QtPrivate::IndexList , int , int ) const>(_a, &QAbstractItemModelSourceAdapter::columnsInserted, 6))
            return;
        if (QtMocHelpers::indexOfMethod<void (QAbstractItemModelSourceAdapter::*)(QtPrivate::IndexList , QAbstractItemModel::LayoutChangeHint )>(_a, &QAbstractItemModelSourceAdapter::layoutChanged, 7))
            return;
    }
    if (_c == QMetaObject::RegisterPropertyMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 1:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QIntHash >(); break;
        case 0:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QList<int> >(); break;
        }
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<QList<int>*>(_v) = _t->availableRoles(); break;
        case 1: *reinterpret_cast<QIntHash*>(_v) = _t->roleNames(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setAvailableRoles(*reinterpret_cast<QList<int>*>(_v)); break;
        default: break;
        }
    }
}

const QMetaObject *QAbstractItemModelSourceAdapter::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QAbstractItemModelSourceAdapter::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN31QAbstractItemModelSourceAdapterE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int QAbstractItemModelSourceAdapter::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 25)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 25;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 25)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 25;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 2;
    }
    return _id;
}

// SIGNAL 0
void QAbstractItemModelSourceAdapter::availableRolesChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void QAbstractItemModelSourceAdapter::dataChanged(QtPrivate::IndexList _t1, QtPrivate::IndexList _t2, QList<int> _t3)const
{
    QMetaObject::activate<void>(const_cast< QAbstractItemModelSourceAdapter *>(this), &staticMetaObject, 1, nullptr, _t1, _t2, _t3);
}

// SIGNAL 2
void QAbstractItemModelSourceAdapter::rowsInserted(QtPrivate::IndexList _t1, int _t2, int _t3)const
{
    QMetaObject::activate<void>(const_cast< QAbstractItemModelSourceAdapter *>(this), &staticMetaObject, 2, nullptr, _t1, _t2, _t3);
}

// SIGNAL 3
void QAbstractItemModelSourceAdapter::rowsRemoved(QtPrivate::IndexList _t1, int _t2, int _t3)const
{
    QMetaObject::activate<void>(const_cast< QAbstractItemModelSourceAdapter *>(this), &staticMetaObject, 3, nullptr, _t1, _t2, _t3);
}

// SIGNAL 4
void QAbstractItemModelSourceAdapter::rowsMoved(QtPrivate::IndexList _t1, int _t2, int _t3, QtPrivate::IndexList _t4, int _t5)const
{
    QMetaObject::activate<void>(const_cast< QAbstractItemModelSourceAdapter *>(this), &staticMetaObject, 4, nullptr, _t1, _t2, _t3, _t4, _t5);
}

// SIGNAL 5
void QAbstractItemModelSourceAdapter::currentChanged(QtPrivate::IndexList _t1, QtPrivate::IndexList _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 5, nullptr, _t1, _t2);
}

// SIGNAL 6
void QAbstractItemModelSourceAdapter::columnsInserted(QtPrivate::IndexList _t1, int _t2, int _t3)const
{
    QMetaObject::activate<void>(const_cast< QAbstractItemModelSourceAdapter *>(this), &staticMetaObject, 6, nullptr, _t1, _t2, _t3);
}

// SIGNAL 7
void QAbstractItemModelSourceAdapter::layoutChanged(QtPrivate::IndexList _t1, QAbstractItemModel::LayoutChangeHint _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 7, nullptr, _t1, _t2);
}
QT_WARNING_POP
